/*******************************************************************************
 * Example for EMG click
 *
 * Date         Mar 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
 * Test configuration PIC32 :
 *  MCU           :        P32MX795F512L
 *  Dev. Board    :        EasyPIC Fusion v7
 *  SW            :        mikroC PRO for PIC32 v3.6.0
 *
 *  NOTES         :
 *                   - Place EMG click board on mikroBUS slot 1
 *                   - Turn switches SW12.3 and SW12.4 in right position
 *                   - Use USB UART B
 *                   - Pull-down PORTA15 turn on Button Press Level
 *                   - Turn on Button Press Level switch SW10 for PORTA&C
 *                   - If the button on A15 is pressed, interrupts are activated 
 *                     ( Use on-click potentiometer for signal calibration )
 *                     ( Set COM port Latency Timer to 1 ms )
 *
*******************************************************************************/

#include <stdint.h>
#include <stdbool.h>

void InitTimer1()
{
  // 300 Hz / 3.33 ms
  T1CON	 = 0x8010;
  T1IP0_bit	 = 1;
  T1IP1_bit	 = 1;
  T1IP2_bit	 = 1;
  T1IF_bit	 = 0;
  T1IE_bit	 = 1;
  PR1		 = 41667;
  TMR1		 = 0;
}

static bool read_flag = false;
static volatile uint32_t interrupt_ctr = 0;
static volatile uint32_t seconds_counter = 0;
uint32_t adc_reads;
double timer_reads;

void main()
{
     uint32_t i = 0;
     char timer_read_string[10];
     char final_string [20];

     TRISB8_bit =  1;              // Set A15 and B8 pins as input
     TRISA15_bit = 1;
     ad1pcfg = 0x0100;
     ADC1_Init();

     UART5_Init( 57600 );
     delay_ms( 500 );

     while(1)
     {

        // If the button on A15 is pressed, interrupts are activated 
        // and the measuring begins 
        if (Button(&PORTA, 15, 10, 1))          
        {
              UART5_Write_Text("START\r\n");
              delay_ms(500);
              InitTimer1();
              EnableInterrupts();
        }

        // Every 3.3 ms measure data and send them to mikroPlot
        if (read_flag == true)                   
        {
          T1IE_bit         = 0;
          adc_reads    =  ADC1_Get_Sample(8);
          timer_reads  =  ((double)interrupt_ctr)* 3.33;
          inttostr(adc_reads, final_string);
          floattostr(timer_reads, timer_read_string);
          strcat(final_string, ",");
          strcat(final_string, timer_read_string);
          ltrim(final_string);
          UART5_Write_Text(final_string);
          UART5_Write_Text("\r\n");
          read_flag = false;
          T1IE_bit  = 1;
        }
     }

}


void Timer1Interrupt() iv IVT_TIMER_1 ilevel 7 ics ICS_SRS
{
    read_flag = true;

    interrupt_ctr++;
    if (interrupt_ctr % 300 == 0)
       seconds_counter++;

    T1IF_bit = 0;
}